import path from 'path'
import fs from 'fs'

const files = [
  'README.md',
]

const resolveDistPath = file =>
  path.resolve(__dirname, '..', 'dist', path.basename(file))

const copyFile = file => {
  const distPath = resolveDistPath(file)
  return new Promise((resolve, reject) => {
    fs.copyFile(
      file,
      distPath,
      err => {
        if (err) reject(err)
        resolve()
      },
    )
  }).then(() => console.log(`Copied ${file} to ${distPath}`))
}

const createPackageFile = () => new Promise((resolve, reject) => {
  fs.readFile(path.resolve(__dirname, '..', 'package.json'), 'utf8', (err, data) => {
    if (err) reject(err)
    resolve(data)
  })
})
  .then(data => JSON.parse(data))
  .then(packageData => {
    const {
      author,
      version,
      description,
      keywords,
      repository,
      license,
      bugs,
      homepage,
      peerDependencies,
      dependencies,
    } = packageData

    const minimalPackage = {
      name: '@speechkit/speechkit-audio-player',
      author,
      version,
      description,
      main: './speechkit-player.js',
      keywords,
      repository,
      license,
      bugs,
      homepage,
      peerDependencies,
      dependencies,
    }

    return new Promise((resolve, reject) => {
      const distPath = path.resolve(__dirname, '..', 'dist', 'package.json')
      const data = JSON.stringify(minimalPackage, null, 2)
      fs.writeFile(distPath, data, err => {
        if (err) reject(err)
        console.log(`Created package.json in ${distPath}`)
        resolve()
      })
    })
  })

const main = () => {
  Promise.all(files.map(copyFile))
    .then(() => {
      createPackageFile()
    })
    .catch(error => console.log(error))
}

main()
